<?php

/**
 * @file
 * This file contains all code for forms and settings regarding the
 * watchdog aggregator module.
 *
 * @author Jens Jahnke <jan0sch@gmx.net>
 */

/**
 * Generate the form for a site configuration.
 *
 * @param array $form_state
 * @param array $site
 * @return array
 */
function watchdog_aggregator_admin_form_add_site($form, &$form_state, $site = array()) {

  $form['sid'] = array(
    '#type' => 'value',
    '#value' => isset($site['sid']) ? $site['sid'] : 0,
  );
  $form['url'] = array(
    '#title' => t('URL'),
    '#description' => t('Please enter the url of your website here (http://example.com).'),
    '#type' => 'textfield',
    '#maxlength' => 254,
    '#required' => TRUE,
    '#default_value' => isset($site['url']) ? $site['url'] : '',
  );
  $form['dbhost'] = array(
    '#title' => t('Database host'),
    '#description' => t('Enter the url of your database server here (db.example.com).'),
    '#type' => 'textfield',
    '#maxlength' => 254,
    '#required' => TRUE,
    '#default_value' => isset($site['dbhost']) ? $site['dbhost'] : '',
  );
  $form['dbport'] = array(
    '#title' => t('Database port'),
    '#description' => t('Enter the port your database server is listening on. You may stick with "0" for the default port.'),
    '#type' => 'textfield',
    '#maxlength' => 5,
    '#required' => TRUE,
    '#default_value' => isset($site['dbport']) ? $site['dbport'] : 0,
  );
  $form['dbtype'] = array(
    '#title' => t('Database type'),
    '#description' => t('Select the type of your database here. Beware that mysqli is the recommended way to connect to MySQL.'),
    '#type' => 'select',
    '#options' => array(
      'mysql' => t('MySQL'),
      'pgsql' => t('PostgreSQL'),
    ),
    '#required' => TRUE,
    '#default_value' => isset($site['dbtype']) ? $site['dbtype'] : 'mysql',
  );
  $form['dbname'] = array(
    '#title' => t('Database name'),
    '#description' => t('Enter the name of your drupal database here.'),
    '#type' => 'textfield',
    '#maxlength' => 254,
    '#required' => TRUE,
    '#default_value' => isset($site['dbname']) ? $site['dbname'] : '',
  );
  $form['dbuser'] = array(
    '#title' => t('Database user'),
    '#description' => t('The username for database access.'),
    '#type' => 'textfield',
    '#maxlength' => 254,
    '#required' => TRUE,
    '#default_value' => isset($site['dbuser']) ? $site['dbuser'] : '',
  );
  $form['dbpass'] = array(
    '#title' => t('Database password'),
    '#description' => t('The password for database access.'),
    '#type' => 'password',
    '#maxlength' => 254,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );


  return $form;
}


/**
 * Submit function for watchdog_aggregator_admin_form_add_site.
 *
 * @param array $form
 * @param array $form_state
 * @return void
 */
function watchdog_aggregator_admin_form_add_site_submit($form, &$form_state) {
  $url = (string)trim($form_state['values']['url']);
  $dbhost = (string)trim($form_state['values']['dbhost']);
  $dbport = (int)trim($form_state['values']['dbport']);
  $dbtype = (string)trim($form_state['values']['dbtype']);
  $dbname = (string)trim($form_state['values']['dbname']);
  $dbuser = (string)trim($form_state['values']['dbuser']);
  $dbpass = (string)trim($form_state['values']['dbpass']);
  $sid = (int)trim($form_state['values']['sid']);
  if ($sid === 0) {
    watchdog_aggregator_add_site($url, $dbhost, $dbport, $dbtype, $dbname, $dbuser, $dbpass);
  }
  else {
    watchdog_aggregator_update_site($sid, $url, $dbhost, $dbport, $dbtype, $dbname, $dbuser, $dbpass);
  }
  $form_state['redirect'] = array('admin/config/watchdog_aggregator/sites');
}

/**
 * Validation function for watchdog_aggregator_admin_form_add_site.
 *
 * @param array $form
 * @param array $form_state
 */
function watchdog_aggregator_admin_form_add_site_validate($form, &$form_state) {
  $url = (string)trim($form_state['values']['url']);
  if (!valid_url($url, TRUE)) {
    form_set_error('url', t('Please enter a valid absolute url!'));
  }
}

/**
 * Generates the settings form for the watchdog aggregator.
 *
 * @param array $form_state
 * @return array
 */
function watchdog_aggregator_admin_form_cache($form_state) {
  $form = array();

  $times = array(
    3600   => t('One hour'),
    7200   => t('Two hours'),
    14400  => t('Four hours'),
    28800  => t('Eight hours'),
    43200  => t('Twelve hours'),
    86400  => t('One day'),
    172800 => t('Two days'),
    259200 => t('Three days'),
    345600 => t('Four days'),
    432000 => t('Five days'),
  );
  $form['cache'] = array(
    '#title' => t('Cache settings'),
    '#type' => 'fieldset',
  );
  $form['cache']['watchdog_aggregator_cache_max_age'] = array(
    '#type' => 'select',
    '#title' => t('Maximum age of cached entries'),
    '#description' => t('All entries in the log cache that are older than this will be deleted on the next cron run. Be aware that log entries that are older than the limit you set here will also not be imported!'),
    '#options' => $times,
    '#default_value' => (int)variable_get('watchdog_aggregator_cache_max_age', 172800),
    '#required' => TRUE,
  );

  $levels = array(
    WATCHDOG_EMERGENCY    => t('Emergency: system is unusable'),
    WATCHDOG_ALERT    => t('Alert: action must be taken immediately'),
    WATCHDOG_CRITICAL => t('Critical: critical conditions'),
    WATCHDOG_ERROR    => t('Error: error conditions'),
    WATCHDOG_WARNING  => t('Warning: warning conditions'),
    WATCHDOG_NOTICE   => t('Notice: normal but significant condition'),
  );
  $form['fetch'] = array(
    '#title' => t('Fetch settings'),
    '#type' => 'fieldset',
  );
  $form['fetch']['watchdog_aggregator_fetch_interval'] = array(
    '#type' => 'select',
    '#title' => t('Fetch intervall'),
    '#description' => t('The watchdog aggregator will query all sites and cache their log data in this intervall.'),
    '#options' => $times,
    '#default_value' => (int)variable_get('watchdog_aggregator_fetch_interval', 86400),
    '#required' => TRUE,
  );
  $form['fetch']['watchdog_aggregator_minimum_log_level'] = array(
    '#type' => 'select',
    '#title' => t('The minimum log level to import.'),
    '#description' => t('Select the minimum log level to import here. Usually you set this to "Critical" or "Alert" to prevent massive downloads of log data.'),
    '#options' => $levels,
    '#default_value' => (int)variable_get('watchdog_aggregator_minimum_log_level', WATCHDOG_ERROR),
    '#required' => TRUE,
  );

  return system_settings_form($form);
}

/**
 * A wrapper for the delete form.
 * 
 * @param int $sid
 * @return string
 */
function watchdog_aggregator_admin_form_delete_callback($sid) {
  return drupal_get_form('watchdog_aggregator_admin_form_delete', $sid);
}

/**
 * Asks if the user really wants to delete the site configuration.
 *
 * @param array $form_state
 * @param int $sid
 * @return array
 */
function watchdog_aggregator_admin_form_delete($form, &$form_state, $sid) {
  $sid = (int)$sid;
  $site = watchdog_aggregator_get_site($sid);
  if (!empty($site)) {
    return confirm_form(
      array(
        'sid' => array('#type' => 'value', '#value' => $site['sid']),
      ),
      t('Are you sure you want to delete the site configuration for %u?', array('%u' => $site['url'])),
      'admin/config/watchdog_aggregator/site/'. $site['sid']
    );
  }
  return array();
}

/**
 * Deletes a site configuration and all of it's log entries.
 *
 * @param array $form
 * @param array $form_state
 */
function watchdog_aggregator_admin_form_delete_submit($form, &$form_state) {
  watchdog_aggregator_delete_site($form_state['values']['sid']);
  $form_state['redirect'] = array('admin/config/watchdog_aggregator/sites');
}

/**
 * Generates the form for site administration.
 *
 * @param array $form_state
 * @return array
 */
function watchdog_aggregator_admin_form_sites($form, &$form_state) {
  
  $sites = watchdog_aggregator_get_sites();
  $stats = array();
  if (!empty($sites)) {
    $form['sites'] = array(
      '#title' => t('Sites'),
      '#type' => 'fieldset',
    );

    $header = array(
      t('Website'),
      t('Fetch logs'),
      t('Last log entry'),
      t('Log entries in cache')
    );
    $rows = array();
    $switch = '';
    foreach ($sites as $site) {
      if ((int)$site['active'] === 0) {
        $switch = t('No');
      }
      else {
        $switch = t('Yes');
      }
      array_push($rows, array(
          l($site['url'], 'admin/config/watchdog_aggregator/site/'. $site['sid'], array('attributes' => array('title' => t('Click here to view details about this configuration or to edit it.')))),
          l($switch, 'admin/config/watchdog_aggregator/site/'. $site['sid'] .'/toggle', array('attributes' => array('title' => t('Click here to toggle the fetch status of this site configuration.')))),
          date('Y-m-d H:i:s O', watchdog_aggregator_get_last_sync($site['sid'])),
          watchdog_aggregator_count_log_lines($site['sid']),
        ));
      $stats[$site['sid']] = watchdog_aggregator_get_statistics($site['sid']);
    }
    $form['sites']['list'] = array(
      '#type' => 'markup',
      '#markup' => theme('table', array('header' => $header, 'rows' => $rows)),
    );
    // Statistics
    $plot = array();
    foreach ($stats as $key => $value) {
      $tmp = array();
      foreach ($value as $eid => $count) {
        $tmp[] = array($eid, $count['count']);
      }
      $p = new flotData($tmp);
      //Flot module bug.
      $p->lines = new stdClass();
      $p->lines->show = FALSE;
      $p->bars = new stdClass();
      $p->bars->show = TRUE;
      $s = watchdog_aggregator_get_site($key);
      $p->label = $s['url'];
      $plot[] = $p;
    }
    $options = array(
      'legend' => array('container' => '#watchdog_aggregator_sites_graph_legend'),
      'xaxis' => array(
        'ticks' => array(
          array(WATCHDOG_EMERGENCY + 0.5, t('Emergency')),
          array(WATCHDOG_ALERT + 0.5, t('Alert')),
          array(WATCHDOG_CRITICAL + 0.5, t('Critical')),
          array(WATCHDOG_ERROR + 0.5, t('Error')),
          array(WATCHDOG_WARNING + 0.5, t('Warning')),
          array(WATCHDOG_NOTICE + 0.5, t('Notice')),
        ),
      ),
      'yaxis' => array('min' => 0, 'minTickSize' => 1),
    );
    $form['stats'] = array(
      '#title' => t('Statistics'),
      '#description' => t('The graph below shows the amount of log entries per type for each site.'),
      '#type' => 'fieldset',
    );
    $form['stats']['graph'] = array(
      '#type' => 'markup',
      '#prefix' => '<div>',
      '#markup' => theme('flot_graph', array('element' => NULL, 'data' => $plot, 'options' => $options)),
      '#suffix' => '</div>',
    );
    $form['stats']['legend'] = array(
      '#type' => 'markup',
      '#value' => '<div id="watchdog_aggregator_sites_graph_legend"></div>',
    );
  }
  else {
    drupal_set_message(t('No sites are defined.'));
  }


  return $form;
}

/**
 * A wrapper callback for editing a site configuration.
 *
 * @param int $sid The id of the site configuration.
 * @return string
 */
function watchdog_aggregator_admin_site_details($sid = 0) {
  $sid = (int)$sid;
  $site = watchdog_aggregator_get_site($sid);
  if (!empty($site)) {
    return drupal_get_form('watchdog_aggregator_admin_form_add_site', $site);
  }
  drupal_set_message(t('No such site configuration!'), 'warning');
  drupal_goto('admin/config/watchdog_aggregator/sites');
}

/**
 * Menu callback; displays a listing of log messages.
 *
 * Messages are truncated at 56 chars. Full-length message could be viewed at
 * the message details page.
 *
 * @ingroup logging_severity_levels
 */
function watchdog_aggregator_overview() {
  $filter = watchdog_aggregator_build_filter_query();
  $rows = array();
  $classes = array(
    WATCHDOG_DEBUG     => 'dblog-debug',
    WATCHDOG_INFO      => 'dblog-info',
    WATCHDOG_NOTICE    => 'dblog-notice',
    WATCHDOG_WARNING   => 'dblog-warning',
    WATCHDOG_ERROR     => 'dblog-error',
    WATCHDOG_CRITICAL  => 'dblog-critical',
    WATCHDOG_ALERT     => 'dblog-alert',
    WATCHDOG_EMERGENCY => 'dblog-emerg',
  );

  $build['watchdog_aggregator_filter_form'] = drupal_get_form('watchdog_aggregator_filter_form');

  $header = array(
    '', // Icon column.
    array('data' => t('Type'), 'field' => 'w.type'),
    array('data' => t('Date'), 'field' => 'w.wid', 'sort' => 'desc'),
    t('Message'),
    array('data' => t('Website'), 'field' => 's.url'),
    array('data' => t('Operations')),
  );

  $query = db_select('watchdog_aggregator_logs', 'w')->extend('PagerDefault')->extend('TableSort');
  $query->leftJoin('watchdog_aggregator_sites', 's', 'w.sid = s.sid');
  $query
    ->fields('w', array('wid', 'sid', 'severity', 'type', 'timestamp', 'message', 'variables', 'link'))
    ->addField('s', 'url');
  if (!empty($filter['where'])) {
    $query->where($filter['where'], $filter['args']);
  }
  $result = $query
    ->limit(50)
    ->orderByHeader($header)
    ->execute();

  foreach ($result as $dblog) {
    $rows[] = array('data' =>
      array(
        // Cells
        array('class' => 'icon'),
        t($dblog->type),
        format_date($dblog->timestamp, 'short'),
        theme('watchdog_aggregator_message', array('event' => $dblog, 'link' => TRUE)),
        l($dblog->url, $dblog->url, array('attributes' => array('target' => '_blank'))),
        filter_xss(str_replace('<a href="', '<a href="' . $dblog->url, $dblog->link)),
      ),
      // Attributes for tr
      'class' => array(drupal_html_class('dblog-' . $dblog->type), $classes[$dblog->severity]),
    );
  }

  $build['dblog_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'admin-dblog'),
    '#empty' => t('No log messages available.'),
  );
  $build['dblog_pager'] = array('#theme' => 'pager');

  return $build;
}

/**
 * Build query for dblog administration filters based on session.
 * Taken from dblog module.
 *
 * @return mixed
 */
function watchdog_aggregator_build_filter_query() {
  if (empty($_SESSION['watchdog_aggregator_overview_filter'])) {
    return;
  }

  $filters = watchdog_aggregator_filters();

  // Build query
  $where = $args = array();
  foreach ($_SESSION['watchdog_aggregator_overview_filter'] as $key => $filter) {
    $filter_where = array();
    foreach ($filter as $value) {
      $filter_where[] = $filters[$key]['where'];
      $args[] = $value;
    }
    if (!empty($filter_where)) {
      $where[] = '('. implode(' OR ', $filter_where) .')';
    }
  }
  $where = !empty($where) ? implode(' AND ', $where) : '';

  return array(
    'where' => $where,
    'args' => $args,
  );
}

/**
 * List wd administration filters that can be applied.
 * Taken from dblog module.
 *
 * @return array
 */
function watchdog_aggregator_filters() {
  $filters = array();

  foreach (_watchdog_aggregator_get_message_types() as $type) {
    $types[$type] = $type;
  }

  if (!empty($types)) {
    $filters['type'] = array(
      'title' => t('Type'),
      'where' => "w.type = ?",
      'options' => $types,
    );
  }

  $filters['severity'] = array(
    'title' => t('Severity'),
    'where' => 'w.severity = ?',
    'options' => watchdog_severity_levels(),
  );
  
  $sites = watchdog_aggregator_get_sites();
  if (!empty($sites)) {
    $ids = array();
    foreach ($sites as $site) {
      $ids[$site['sid']] = $site['url'];
    }
    $filters['site'] = array(
      'title' => t('Website'),
      'where' => 'w.sid = ?',
      'options' => $ids,
    );
  }

  return $filters;
}

/**
 * Return form for dblog administration filters.
 *
 * @ingroup forms
 * @see watchdog_aggregator_filter_form_submit()
 * @see watchdog_aggregator_filter_form_validate()
 */
function watchdog_aggregator_filter_form($form) {
  $filters = watchdog_aggregator_filters();

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter log messages'),
    '#collapsible' => TRUE,
    '#collapsed' => empty($_SESSION['watchdog_aggregator_overview_filter']),
  );
  foreach ($filters as $key => $filter) {
    $form['filters']['status'][$key] = array(
      '#title' => $filter['title'],
      '#type' => 'select',
      '#multiple' => TRUE,
      '#size' => 8,
      '#options' => $filter['options'],
    );
    if (!empty($_SESSION['watchdog_aggregator_overview_filter'][$key])) {
      $form['filters']['status'][$key]['#default_value'] = $_SESSION['watchdog_aggregator_overview_filter'][$key];
    }
  }

  $form['filters']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['filters']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );
  if (!empty($_SESSION['watchdog_aggregator_overview_filter'])) {
    $form['filters']['actions']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset')
    );
  }

  return $form;
}

/**
 * Validate result from dblog administration filter form.
 */
function watchdog_aggregator_filter_form_validate($form, &$form_state) {
  if ($form_state['values']['op'] == t('Filter') && empty($form_state['values']['type']) && empty($form_state['values']['severity'])) {
    form_set_error('type', t('You must select something to filter by.'));
  }
}

/**
 * Process result from dblog administration filter form.
 */
function watchdog_aggregator_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = watchdog_aggregator_filters();
  switch ($op) {
    case t('Filter'):
      foreach ($filters as $name => $filter) {
        if (isset($form_state['values'][$name])) {
          $_SESSION['watchdog_aggregator_overview_filter'][$name] = $form_state['values'][$name];
        }
      }
      break;
    case t('Reset'):
      $_SESSION['watchdog_aggregator_overview_filter'] = array();
      break;
  }
  return 'admin/reports/watchdog_aggregator';
}

/**
 * Shows a single event in detailed form.
 *
 * @param int $wid The watchdog log ID.
 * @return string A HTML-Page.
 */
function watchdog_aggregator_show_event($id) {
  $severity = watchdog_severity_levels();
  $result = db_query("SELECT * FROM {watchdog_aggregator_logs} WHERE wid = :wid", array('wid' => $id))->fetchObject();
  if ($dblog = $result) {
    $site = watchdog_aggregator_get_site($dblog->sid);
    $rows = array(
      array(
        array('data' => t('Type'), 'header' => TRUE),
        t($dblog->type),
      ),
      array(
        array('data' => t('Date'), 'header' => TRUE),
        format_date($dblog->timestamp, 'long'),
      ),
      array(
        array('data' => t('Website'), 'header' => TRUE),
        l($site['url'], $site['url'], array('attributes' => array('target' => '_blank'))),
      ),
      array(
        array('data' => t('Location'), 'header' => TRUE),
        l($dblog->location, $dblog->location),
      ),
      array(
        array('data' => t('Referrer'), 'header' => TRUE),
        l($dblog->referer, $dblog->referer),
      ),
      array(
        array('data' => t('Message'), 'header' => TRUE),
        theme('watchdog_aggregator_message', array('event' => $dblog)),
      ),
      array(
        array('data' => t('Severity'), 'header' => TRUE),
        $severity[$dblog->severity],
      ),
      array(
        array('data' => t('Hostname'), 'header' => TRUE),
        check_plain($dblog->hostname),
      ),
      array(
        array('data' => t('Operations'), 'header' => TRUE),
        $dblog->link,
      ),
    );
    $build['dblog_table'] = array(
      '#theme' => 'table',
      '#rows' => $rows,
      '#attributes' => array('class' => array('dblog-event')),
    );
    return $build;
  }
  else {
    return '';
  }
}

/**
 * Returns HTML for a log message.
 *
 * @param $variables
 *   An associative array containing:
 *   - event: An object with at least the message and variables properties.
 *   - link: (optional) Format message as link, event->wid is required.
 *
 * @ingroup themeable
 */
function theme_watchdog_aggregator_message($variables) {
  $output = '';
  $event = $variables['event'];
  // Check for required properties.
  if (isset($event->message) && isset($event->variables)) {
    // Messages without variables or user specified text.
    if ($event->variables === 'N;') {
      $output = $event->message;
    }
    // Message to translate with injected variables.
    else {
      $output = t($event->message, unserialize($event->variables));
    }
    if ($variables['link'] && isset($event->wid)) {
      // Truncate message to 56 chars.
      $output = truncate_utf8(filter_xss($output, array()), 56, TRUE, TRUE);
      $output = l($output, 'admin/reports/watchdog_aggregator/event/' . $event->wid, array('html' => TRUE));
    }
  }
  return $output;
}
